@@ -23,6 +23,53 @@ Most notably:
 
 from copy import deepcopy
 import operator
+from itertools import izip
+
+#################################
+## Nested structure conversion ##
+#################################
+
+def totuple(a):
+    '''Makes tuples out of nested datastructures like lists and arrays.
+       Authored by Bi Rico, http://stackoverflow.com/questions/10016352/convert-numpy-array-to-tuple'''
+    try:
+        return tuple(totuple(i) for i in a)
+    except TypeError: # dig until we can dig no more!
+        return a
+
+def makeTuple(a):
+    '''Like totuple, but ensures that you get a tuple out.'''
+    retVal = totuple(a)
+    return ( retVal if retVal.__class__==tuple else (retVal,) )
+
+def tolist(a):
+    '''Makes lists out of nested datastructures like tuples,lists, and arrays.
+       Based on totuple.'''
+    try:
+        return [ tolist(i) for i in a ]
+    except TypeError:
+        return a
+
+def iterToX(f,iterable):
+    '''Generalized version of tolist/totuple.
+       Replace all iterables in a nested structure with another type (X) using
+       the constructor function "f".
+       "f" must take an iterable as it's argument.'''
+    try:
+        return f([iterToX(f,i) for i in iterable])
+    except TypeError:
+        return iterable
+
+def iterToX_splat(f,iterable):
+    '''Generalized version of tolist/totuple.
+       Replace all iterables in a nested structure with another type using
+       the constructor function "f" that takes multiple arguments.
+       "f" must take multiple arguments, like sympy.Tuple, for example.'''
+    try:
+        return f(*[iterToX_splat(f,i) for i in iterable])
+    except TypeError:
+        return iterable
+
 
 ###############################
 ## Some basic list utilities ##
@@ -34,9 +81,13 @@ def flatten(l,repetitions=1):
        Use like: flatten(aList,3)
        
        Another (nominally faster) version of this is: [ l for i in aList for j in i for k in j for l in k ]
-       but it's unable to deal with jagged arrays...
+       but it's unable to deal with jagged arrays.
        
-       A terse but slower version of flatten (1 repetition) can also be acheived with: sum(aList,[])'''
+       A terse but slower version of flatten (1 repetition) can also be acheived with: sum(aList,[])
+       
+       An even faster version of flatten (1 repetition, non-jagged arrays)
+       can be acheived with itertools.chain: list(chain(*l))
+       '''
     retVal = l
     for i in range(repetitions):
         gen = ( ( k if hasattr(k,'__iter__') else [k] ) for k in retVal ) # Makes the function able to deal with jagged arrays as well
@@ -44,19 +95,42 @@ def flatten(l,repetitions=1):
     return retVal
 
 def zipflat(*args):
-    '''Like zip, but flattens the result'''
-    return [j for i in zip(*args) for j in i]
+    '''Like zip, but flattens the result.
+       (itertools.izip is used internally for speed/memory usage)'''
+    return [j for i in izip(*args) for j in i]
 
 def ziptranspose(l):
     '''Tranpose the two outer dimensions of a nest list
-       ( This is just a wrapper around zip(*l) )'''
+       (This is just a wrapper around zip(*l))'''
     return zip(*l)
 
+def intersperse(l,separator):
+    '''Intersperse a separator between all items
+       
+       Example:
+       >>> interperse([1,2,3,4],42)
+       [1,42,2,42,3,42,4]
+       
+       Works by zipping together the list and a list of separators, then
+       flattening the result and clipping off the end:
+         flatten(zip( l, [sep]*len(l) ))[:-1]
+       
+       With itertools, this could be written as the longer (but ~3x faster):
+         list(islice(chain(*izip( l, [sep]*len(l) )),0,2*len(l)-1))
+       
+       This horrific list comprehension is also faster:
+         [ k for j in izip( l, [sep]*len(l) ) for k in j ][:-1]'''
+    
+    return flatten(zip( l, [separator]*len(l) ))[:-1]
+
 def removeDuplicates(l):
-    '''Order preserving duplicate removal... automatically converts lists and arrays (which are unhashable) to nested tuples.
-       Modified version of code found here: http://stackoverflow.com/questions/480214/how-do-you-remove-duplicates-from-a-list-in-python-whilst-preserving-order'''
+    '''Order preserving duplicate removal.
+       Automatically converts lists and arrays (which are unhashable) to nested tuples.
+       Modified version of code found here:
+       http://stackoverflow.com/questions/480214/how-do-you-remove-duplicates-from-a-list-in-python-whilst-preserving-order'''
     seen = set()
-    return [ x for x in totuple(l) if x not in seen and not seen.add(x)]
+    return [ x for x in totuple(l)
+               if x not in seen and not seen.add(x) ]
 
 def removeAdjacentDuplicates(l):
     '''Replace any groups of items with the same value with a single occurrence instead.
@@ -66,7 +140,8 @@ def removeAdjacentDuplicates(l):
                   if l[i]!=l[i+1] ] + l[-1:]
 
 def deletecases(l,cases):
-    '''Delete all elements of list "cases" from list "l"'''
+    '''Delete all elements of list "cases" from list "l"
+       For large numbers of cases, pass cases as a set for speed'''
     if not hasattr(cases,'__iter__'):
         cases=[cases]
     return [ i for i in l if i not in cases ]
@@ -112,6 +187,15 @@ def groupByFunction(l,f,appendFun=None):
        
        This method is based on an example given for collections.defaultdict in:
        http://docs.python.org/2/library/collections.html
+       
+       Another simple way to acheive this would be using itertools.groupby:
+       
+       from itertools import groupby, imap
+       if appendFun==None:
+           return { k:list(v) for k,v in groupby(l,f) }
+        else:
+            return { k:list(imap(appendFun,v)) for k,v in groupby(l,f) }
+        I may change this in a future version after testing it more.
        '''
     groupDict = {}
     for i in l:
@@ -194,33 +278,18 @@ def interp(l,index):
         indexB = indexA + (1 if index>=0 else -1)
         return l[indexA]*(1-m) + l[indexB]*(m)
 
+def assertSameAndCondense(l,message='List values differ!'):
+    '''Take a list of values that should all be the same, assert that this is true,
+       and then return the common value
+       This acts as a safe funnel in exploratory data processing,
+       cutting a large same-valued list down to a single value.'''
+    assert all([ i==l[0] for i in l ]),message
+    return l[0]
+
 ##########################################
 ## Some utilities for nested structures ##
 ##########################################
 
-def totuple(a):
-    '''Makes tuples out of nested datastructures like lists and arrays.
-       Authored by Bi Rico, http://stackoverflow.com/questions/10016352/convert-numpy-array-to-tuple'''
-    try:
-        return tuple(totuple(i) for i in a)
-    except TypeError: # dig until we can dig no more!
-        return a
-
-def makeTuple(a):
-    '''Like totuple, but ensures that you get a tuple out.'''
-    retVal = totuple(a)
-    return ( retVal if retVal.__class__==tuple else (retVal,) )
-
-def iterToX(f,iterable):
-    '''Replace all iterables in a nested structure (list,tuple,etc...)
-       with another type "f" (actually just any function returning iterables)
-       Like totuple, but more general, also uses list comprehensions
-       instead of generators for more generality (notably sympy.Tuple)'''
-    try:
-        return f(*[iterToX(f,i) for i in iterable])
-    except TypeError: # dig until we can dig no more!
-        return iterable
-
 def getMaxDepth(l,depth=0):
     '''Get the maximum depth of any nested structure.
         For a numpy array, this is the same as ndim.'''
@@ -335,6 +404,46 @@ def deepAdd(x,y):
 def deepMul(x,y):
     return applyInfix_DeepCompare(operator.mul,x,y)
 
+def listAdd(x,y,matchTopDown=True):
+    '''matchTopDown=True uses ShallowCompare
+       matchTopDown=False uses DeepCompare (slower)'''
+    _applyInfix = ( applyInfix_ShallowCompare if matchTopDown else
+                   applyInfix_DeepCompare )
+    return _applyInfix(operator.add,x,y)
+
+def listSub(x,y,matchTopDown=True):
+    '''matchTopDown=True uses ShallowCompare
+       matchTopDown=False uses DeepCompare (slower)'''
+    _applyInfix = ( applyInfix_ShallowCompare if matchTopDown else
+                   applyInfix_DeepCompare )
+    return _applyInfix(operator.sub,x,y)
+
+def listMul(x,y,matchTopDown=True):
+    '''matchTopDown=True uses ShallowCompare
+       matchTopDown=False uses DeepCompare (slower)'''
+    _applyInfix = ( applyInfix_ShallowCompare if matchTopDown else
+                   applyInfix_DeepCompare )
+    return _applyInfix(operator.mul,x,y)
+
+def listDiv(x,y,matchTopDown=True,useIntegerDivision=False):
+    '''matchTopDown=True uses ShallowCompare
+       matchTopDown=False uses DeepCompare (slower)
+       
+       useIntegerDivision=True uses interger division (//)
+       instead of true division'''
+    _applyInfix = ( applyInfix_ShallowCompare if matchTopDown else
+                   applyInfix_DeepCompare )
+    _div = ( operator.div if useIntegerDivision else
+             operator.truediv )
+    return _applyInfix(_div,x,y)
+
+def listIncr(x):
+    return listAdd(x,1)
+
+def listDecr(x):
+    return listAdd(x,-1)
+
+
 def interpGen(l,index):
     '''Just like interp except that it uses the generic shallowAdd and shallowMul.'''
     m = index % 1
@@ -350,23 +459,32 @@ class fancyIndexingList(list):
     '''fancyIndexingList is overloaded as "fL"
        Gives lists the magical properties of numpy arrays, but without requiring regular shapes...
        Also use it where you would convert a list to a numpy array and back again like: np.array(l)[:,3].tolist()
-       Use like: fl(ANY_LIST)[<any valid numpy slice>]
+       Use like: fL(ANY_LIST)[<any valid numpy slice>]
        Examples:
-         fl( [[1],[2,3],[4,5,6],[7,8,9,10]] )[:,0]  ->  [1, 2, 4, 7]
-         fl( [[1],[2,3],[4,5,6],[7,8,9,10]] )[2:4,0]  ->  [4, 7]
-         fl( [1,[2,3],[[4,5],[6,7]]] )[2,0,1]  ->  5
-       fl also has the extra feature of being able to use lists-within-lists when indexing
+         fL( [[1],[2,3],[4,5,6],[7,8,9,10]] )[:,0]  ->  [1, 2, 4, 7]
+         fL( [[1],[2,3],[4,5,6],[7,8,9,10]] )[2:4,0]  ->  [4, 7]
+         fL( [1,[2,3],[[4,5],[6,7]]] )[2,0,1]  ->  5
+       fL also has the extra feature of being able to use lists-within-lists when indexing
        Examples:
-         fl( [[1,7],[2,3],[4,5,6],[7,8,9,10]] )[:,(0,1)]  ->  [[1, 7], [2, 3], [4, 5], [7, 8]]
-         fl( [[1,7],[2,3],[4,5,6],[7,8,9,10]] )[(0,2),(0,1)]  ->  [[1, 7], [4, 5]]
+         fL( [[1,7],[2,3],[4,5,6],[7,8,9,10]] )[:,(0,1)]  ->  [[1, 7], [2, 3], [4, 5], [7, 8]]
+         fL( [[1,7],[2,3],[4,5,6],[7,8,9,10]] )[(0,2),(0,1)]  ->  [[1, 7], [4, 5]]
        Beware that you will need to nest lists or tuples if you only have a single index:
-         fl([1,2,3,4,5])[(1,2)] -> TypeError -- this is equivalent to fl([1,2,3,4,5])[1,2]
-         fl([1,2,3,4,5])[[1,2]] -> TypeError -- ""
-         fl([1,2,3,4,5])[(1,2),] -> [2,3]
-         fl([1,2,3,4,5])[[1,2],] -> [2,3]
-         fl([1,2,3,4,5])[((1,2),)] -> [2,3]
-         fl([1,2,3,4,5])[([1,2],)] -> [2,3]
-         fl([1,2,3,4,5])[[[1,2]]] -> [2,3]
+         fL([1,2,3,4,5])[(1,2)] -> TypeError -- this is equivalent to fl([1,2,3,4,5])[1,2]
+         fL([1,2,3,4,5])[[1,2]] -> TypeError -- ""
+         fL([1,2,3,4,5])[(1,2),] -> [2,3]
+         fL([1,2,3,4,5])[[1,2],] -> [2,3]
+         fL([1,2,3,4,5])[((1,2),)] -> [2,3]
+         fL([1,2,3,4,5])[([1,2],)] -> [2,3]
+         fL([1,2,3,4,5])[[[1,2]]] -> [2,3]
+       
+       And, fL indices can also be nested; this gives a new list which has the (0,0) element and the (2,1) element:
+         fL( [[1,7],[2,3],[4,5,6],[7,8,9,10]] )[((0,0),(2,1)),] -> [1,5]
+       And in case your head doesn't hurt by now (mine does), here is an example that combines all of the above,
+       indexing a 2x2x2x2 square list:
+         fL( [[[[1,2],[3,4]],[[5,6],[7,8]]],[[[9,10],[11,12]],[[13,14],[15,16]]]] )[:, ((0,0),(0,1),1), 1]
+          -> [[2, 4, [7,8]], [10, 12, [15,16]]]
+       This type of usage is NOT recommended, because it is so opaque and covoluted.
+       (It's actually just a consequence of the implementation.)
        '''
     def new_fL(self,*args,**kwds):
         '''Just a wrapper around the class constructor for a new instance, fancyIndexingList()'''
@@ -423,3 +541,78 @@ class fancyIndexingListM1(fancyIndexingList):
         '''Just a wrapper around list.__getitem__(), but subtracts 1 from index'''
         return list.__getitem__(self,self.m1gen(index)) # for a single index, use list's default indexing
 fLm1 = fancyIndexingListM1
+
+def genericSlice(length,start=None,stop=None,step=None,includeStop=False,oneBased=False,checkBounds=False):
+    '''A generic version of "slice" supporting:
+       * dropping or keeping the stop value
+       * 0-based and 1-based indexing
+       * optional bounds checking
+       * +/-/None indexes (as usual)
+       
+       Requires the length of the list that will be sliced.
+       
+       Returns a slice object.
+       (failure results in a None value at the moment)'''
+    maxStop = (length-1 if includeStop else length)
+    
+    # Always fail if step is 0
+    if step==0:
+        return None
+    
+    # Fail on 0-values in 1-based indexing when checking bounds
+    if checkBounds and oneBased and (start==0 or stop==0):
+        return None
+    
+    # Wrap negative values to positive ones and then convert from
+    # 1-based to 0-based indexing
+    if start!=None:
+        if start<0:
+            start += length
+        elif oneBased:
+            start -=1
+    if stop!=None:
+        if stop<0:
+            stop += length
+        elif oneBased:
+            stop -=1
+    
+    # If either value goes negative and we are not checking bounds,
+    # then set it to None avoid wrap-around
+    if not checkBounds:
+        if start<0:
+            start=None
+        if stop<0:
+            stop=None
+    
+    # Handle all the None cases
+    if step==None:
+        step=1
+    if step>0:
+        if start==None:
+            start = 0
+        if stop==None:
+            stop = maxStop
+    else:
+        if start==None:
+            start = length-1
+        if stop==None:
+            stop = (0 if includeStop else -1)
+    
+    # Fail on out-of-bounds if checkBounds is specified (still somewhat lenient)
+    if checkBounds:
+        if ( not 0<=start<length or
+             not 0<=stop<length or
+             step>0 and start-stop>1 or
+             step<0 and stop-start>1 ):
+            return None
+    
+    # Increment (or decrement) the stop value to include it
+    if includeStop:
+        # Add the sign of the step to the stop
+        stop += (1 if step>0 else -1)
+        
+    # If this made stop negative, set it to None avoid wrap-around
+    if stop<0:
+        stop=None
+    
+    return slice(start,stop,step)
